(module gallery mzscheme
  
  ;; README
  ;; This will generate a gallery from the images and xml files
  ;; in this directory.  Use by the following:
  ;;
  ;;  1. Add a String <name> to (names)
  ;;  2. Add an image <name>.jpg and entry <name>.xml.
  ;;     Entry has the structure:
  ;;       Entry ::= (entry (name String) (description String))

  (require 
   (lib "list.ss")
   (lib "file.ss")
   (lib "xml.ss" "xml"))

  (provide create-entries   ; -> Xexpr
	   )

  (define (path->bytes x) (string-append "Gallery/" x)) ;;todo
  
  ;; -> Xexpr
  (define (create-entries)
    `(table ([cellpadding "10"])
            (tbody () ,@(map create-entry (read-names)))))
  
  ;; An entry is
  ;; (entry (name String) (description (union String Xexpr)))
  ;; Example
  (define EXAMPLE-ENTRY
    `(entry
      (name "Jeffrey Palm")
      (description "this is an example")))
  
  ;; String -> Xexpr
  ;; TODO: This is pretty disgusting!!!
  (define (create-entry name)
    (printf "     ... ~a~n" name)
    (let* ([file (format "~a.xml" name)]
           [img  (format "~a.jpg" name)]
           [x (read-xml/element (open-input-file file))]
           [y (xml->xexpr x)]
           [z (if (and (pair? y) (eq? (car y) 'entry))
                  (let* ([r (cdddr y)]
                         [n (car r)]
                         [d (caddr r)])
                    (if (and (pair? n) (eq? (car n) 'name))
                        (let ([name (caddr n)])
                          (if (and (pair? d) (eq? (car d) 'description))
                              (let ([desc (caddr d)])
                                `(tr
                                  (td
				   (a ([href ,(format "Gallery/~a" img)])
				      (img ([alt ,name] 
					    [border "0"]
					    [width "100"] 
					    [src ,(format "Gallery/~a" img)]))))
                                  (td ([width "300"])
                                      (strong ,name)
				      (div ,desc)))))))))])
      (xexpr->xml z)))
  
  ;; -> (listof String)
  ;; names of xml files sorted by date modified
  (define (read-names)
    ;; me first!
    (let ([me "jeffpalm"])
      (cons me
	    (let* ([dir (directory-list)]
		   [xml (filter (lambda (x) 
				  (and (not (regexp-match me (path->bytes x)))
				       (regexp-match "\\.xml$" (path->bytes x)))) 
				dir)]
		   [srt (quicksort xml string<?)]
		   [names (map (lambda (x) 
				 (substring x 0 (- (string-length x) 4)))
			       srt)]
		   )
	      names))))
  
  )
