(module blog-aux mzscheme
   
  (require (lib "list.ss")
           (lib "file.ss")
           (lib "date.ss")
           (lib "xml.ss" "xml")
           (file "rdf.ss")
           )

  (provide 
   blog ;; write-normal-page(TODO) String String String -> (listof Element(XML) Element(XML))
   )
  
  (define (path->bytes x) (string-append "Blog/" x)) ;;todo

  ;; write-normal-page(TODO) String String String -> (listof XML XML)
  ;; create a single [div] from the blogs in the Blog/ directory
  (define (blog write-normal-page link title description)
    (parameterize ([current-directory "Blog"]) 
      (let* ([dir (directory-list)]
             [xml (filter (lambda (x) (regexp-match "\\.xml$" (path->bytes x))) dir)]
             [dts (map (lambda (x)
                         (list x (file-or-directory-modify-seconds x))) 
                       xml)]
             [srt (quicksort dts (lambda (x y) (> (cadr x) (cadr y))))]
             [txt (map (lambda (x)
                         (list (with-input-from-file (car x) read-xml/element)
                               (cadr x)))
                       srt)]
             ;; (listof String String[date] Number[archive])
             [fnl (map (lambda (x)
                             (list (car x)
                                   (date->string (seconds->date (cadr x)) #t)))
                       txt)]
             [accum (length txt)] ;; is there a better way to do this
             [items (map (lambda (x)
                           (let ([n accum])
                             (set! accum (- accum 1))
                             (create-item! x n write-normal-page link)))
                         fnl)]
             ;; the xml body of blog.html
             [xml (xexpr->xml
                   `(div () 
                         (h3 "Important Messages from the " (plural-maybe "Instructor"))
                         (a ([href "index.xml"] [style "text-decoration: none"]) 
			    "Syndicate this site (XML)")
                         ,@(map format-blog fnl)))]
             ;; the xml body of index.rdf
             [rdf (xexpr->xml (create-rdf items link title description))])
             (list xml rdf))))
  
  ;; (list Element String) String -> RDFItem
  ;; side effect: writes out an archive to ../Blog-<num>.html
  (define (create-item! x num write-normal-page link) 
    (let* ([ele       (car x)]
           [dat       (cadr x)]
           [div       (format-blog x)]
           [file-name (format "~a.html" (pad num))]
           [file-path (format "../Blog-~a" file-name)]
           [title     (format "Entry ~a" num)]
           [creator   ""])
      ;; create an archive page with num
      (write-normal-page file-path title #f div)
      (make-rdfitem (format "~a/../Blog-~a" link file-name)
                    dat
                    ele
                    title
                    creator)))
  
  ;; (list Element String Number) -> Xexpr[div]
  (define (format-blog x)
    (let ([ele (car x)]
          [dat (cadr x)])
      `(div ()
            (hr)
            (p (b ,dat))
            (blockquote ,(xml->xexpr ele)))))
  
  ;; Number -> String
  (define (pad n)
    (letrec ([pad-accum (lambda (s)
			  (if (>= (string-length s) 10) s
			      (pad-accum (string-append "0" s))))])
      (pad-accum (number->string n))))
  
  )
