(module rdf mzscheme
  
  (require (lib "list.ss")
           (lib "date.ss"))
  
  (provide add-rdf-info     ; Xexpr[html] String -> Xexpr[html]
           create-rdf       ; RDFItems String String String -> Xexpr[rdf:RDF]
           make-rdfitem     ; String[path] String[date] String String String -> RDFItem
           rdfitem-link     ; RDFItem -> String[path]
           rdfitem-date     ; RDFItem -> String[date]
           rdfitem-content  ; RDFItem -> String
           rdfitem-title    ; RDFItem -> String
           rdfitem-creator  ; RDFItem -> String
           )
  
  ;; RDFItem = (listof link date content title creator)
  ;; where all fields are Strings
  
  (define rdfitem-link car)
  (define rdfitem-date cadr)
  (define rdfitem-content caddr)
  (define rdfitem-title cadddr)
  (define (rdfitem-creator x) (car (cadddr x)))
  (define make-rdfitem list)
  
  ;; Xexpr[html] String -> Xexpr[html]
  ;; adds RDF info to the head attribute
  ;; main is a url without a trailing /
  (define (add-rdf-info xml main)
    (let* ([head (cadr xml)]
           [body (cddr xml)]
           [new-head (append 
		      head
		      `((link 
			 ([rel "alternate"] 
			  [type "application/rss+xml"]
			  [title "RSS 2.0"]
			  [href ,(format "~a/index.xml" main)]) "")
			(link 
			 ([rel "alternate"] 
			  [type "application/rss+xml"]
			  [title "RSS"]
			  [href ,(format "~a/index.xml" main)]) "")))])
      `(html ,new-head ,@body)))
  
  (define (create-rdf items link title description)
    `(rss ([version "2.0"])
          (channel 
	   (title ,title)
	   (link ,link)
	   (sy:updatePeriod "hourly")
	   (sy:updateFrequency "1")
	   (description ,description)
	   (lastBuildDate ,(date->string (seconds->date (current-seconds))))
	   ,@(map (lambda (x)
		    (let ([url (car         x)]
			  [dat (cadr        x)]
			  [ele (caddr       x)]
			  [tit (cadddr      x)])
		      `(item
			(title ,tit)
			(description ,ele)
			(link ,url)
			(guid ,url)
			(pubDate ,dat))))
		  items))))
  )
