(module util mzscheme
  
  (require
     (lib "list.ss") )
  
  (provide staff-list->comma-string
           plural-maybe
           number->cardinal
           )
  
    ;; Listof[staff] -> String
  ;; a staff is (cons X Y ...)
  (define (staff-list->comma-string lst)
    (cond
      [(empty? lst) empty]
      [(empty? (cdr lst)) (list (cadar lst))]
      [else (cons (cadar lst) (map (lambda (x) (format ", ~a" (cadr x))) (cdr lst)))]))
  ;; Tests
  (equal? (staff-list->comma-string empty) empty)
  (equal? (staff-list->comma-string '((name1 "Name1")                                )) (list "Name1"))
  (equal? (staff-list->comma-string '((name1 "Name1") (name2 "Name2")                )) (list "Name1, Name2"))
  (equal? (staff-list->comma-string '((name1 "Name1") (name2 "Name2") (name3 "Name3"))) (list "Name1, Name2, Name3"))
  
  ;; (List | Number) -> Number
  ;; Returns the size of x if x is a list, else x
  (define (get-size x)
    (cond
     [(cons? x) (length x)]
     [(number? x) x]
     [else (error "Invalid thingy")]))
  
  ;; String -> String
  ;; attaches appropriate ending to make s a plural
  (define (pluralize s)
    (string-append s "s"))
  
  ;; String (List | Number) -> String
  ;; makes s plural if necessary
  (define (plural-maybe s n)
    (cond
      [(eq? (get-size n) 1) s]
      [else (pluralize s)]))
  
  ;; (List | Number) -> String
  ;; cardinal number for n
  (define (number->cardinal n)
    (vector-ref '#("none" "one" "two" "three" "four" "five" "six" "seven")
                (get-size n)))
  
  )