(module web-page mzscheme 

  (require
    (lib "list.ss")
    (lib "xml.ss" "xml")
    (lib "date.ss") ;new
    (lib "process.ss"))
  
  (provide produce-page html image checked-links remove-link all-pages-produced? LINE)
  
;  (define-values (string->path2 path->string2)
;  (if (regexp-match "299" (version))
;      (values string->path path->string)
;      (values (lambda (x) x) (lambda (x) x)))) 
  
  ; ---------------------------------------------------------------------------
  ; NAVIGATION LINKS, IMAGE LINKS 
  
  ; TYPE Nav-Link = (list (union String Xexpr) (union #f String[url]))

  ; Nav-Link : makes a blank line in a navigation bar
  (define LINE '(nbsp #f))

  ;  (listof Nav-Link) -> Xexpr[table]
  ; a navigation bar from links
  (define (navigation  links)
    `(table ([border "0"][background ,BACKGROUND-MOTIF])
       ,@(map (lambda (a-link)
		(let* ([url (cadr a-link)]
		       [raw (car a-link)]
		       [txt (if (string? raw) 
				`(font ([color ,NAV-BGR-COLOR]) (b ,raw))
				; `(font ([color ,NAV-TXT-COLOR]) (b ,raw))
				raw)])
		  `(tr 
		     (td ,(if url
			      `(a ([href ,url]
				   [style "text-decoration: none"])
				  ,txt)
			      `(font ([color ,NAV-TXT-COLOR]) (b ,raw)))))))
	   links)))

  ; ---------------------------------------------------------------------------
  ; String (listof Nav-Link) - (listof Nav-Link)
  ; remove the first link that has the same "title" as the-link
  (define (remove-link the-link links)
    (cond
      [(pair? links) 
       (if (and (string? (caar links)) (regexp-match (caar links) the-link))
	   (cons `(,(caar links) #f) (cdr links))
           (cons (car links) (remove-link the-link (cdr links))))]
      [else '()]))

  ; ---------------------------------------------------------------------------
  ; pages to be produced

  ; table : (listof String[local url])
  ; keep track of all pages that a script is supposed to create
  (define table '())

  ; String[file name] Xexpr[html] -> Void
  ; effect: create or overwrite file.html with page, remove from table
  (define (produce-page file.html page)
    (printf " ... ~s~n" file.html)
    (set! table (remove file.html table))
    (with-output-to-file file.html 
      (lambda () (write-xml/content (xexpr->xml page)))
      'truncate))

  ; (listof Nav-Link) -> (listof Nav-Link)
  (define (checked-links l) 
    (map (lambda (x) `(,(car x) ,(check-link (cadr x)))) l))

  ; String[page] -> String
  ; effect: register file link with link table
  ; warning: do nothing on http links, check one day
  (define (check-link url)
    (unless (or (not url)
		(regexp-match "^../" url)
		(regexp-match "^mailto:" url)
		(regexp-match "^http:" url))
      (set! table (cons url table)))
    url)

  ; -> Boolean
  ; check whether all promised pages were produced 
  (define (all-pages-produced?)
    (if (null? table)
	(begin
	  (printf " All pages produced.~n")
	  #t)
	(begin
	  (printf
	    " The following pages were supposed to be created but weren't:~n")
	  (for-each (lambda (x) (printf "   ~a~n" x)) table)
	  #f)))
    
  ; ---------------------------------------------------------------------------
  ; String  String (listof Nav-Link) Xexpr . (list Number) -> Xexpr[html]
  ; general page layout; navigation bar on the left, generated from links

  (define (html title logo:url links b . nav)
    (let ([nav-width (if (pair? nav) (car nav) 15)])
      `(html 
	 (head (title ,title))
	 (body ([bgcolor ,BACKGROUND-COLOR][background ,BACKGROUND-MOTIF])
	   (br)
	   (hr ([width "80%"][height "3"]))
	   (table ([align "center"] 
		   [width "80%"]    
		   [border "0"][cellspacing "0"][cellpadding "0"])
	     (tr (td ([width ,(format "~a%" nav-width)]
		      [valign "top"] [align "left"]
		      [rowspan "2"])
		     ,(navigation links))
		 (td ([width "3%"]))
		 (td ([valign "top"][align "left"]) 
		     (h3 ,title))
		 ,(if logo:url
		      `(td ([valign "top"][align "right"]) 
			   (img {[src ,logo:url][alt "logo"]}))
		      'nbsp))
	     (tr (td)
		 (td ([valign "top"][align "left"][colspan "3"]) 
		     (div ,b))))
	   (hr ([width "80%"][height "3"]))
	   (table ([width "80%"][align "center"])
	     (tr (td ([align "left"])
		     (font ([size "-2"])
		       "last updated on " ,today))
		 (td ([align "right"])
		     (font ([size "-2"])
		       "generated with "
		       (a ([href "http://www.plt-scheme.org/"]) 
			  "PLT Scheme")
		       " -- ("
		       (a ([href "csu211-src.tar.gz"]) "Source")
		       ")"))))
	   ))))

  ; ---------------------------------------------------------------------------
  ; Finding images
  
  (define IMAGES "Images/")
  

  ; String Symbol *-> String[url]
  (define (image name . path)
    (printf "path~n")
    ;;todo(path->string (apply build-path (append (cons "./" path) (list IMAGES name)))))
    (string-append "./" path IMAGES name))
  
  (define BACKGROUND-COLOR "#e5e5e5")
  (define BACKGROUND-MOTIF "http://www.ccs.neu.edu/home/matthias/Images/sandston.gif")
  (define NAV-BGR-COLOR    "#9900CC")
  (define NAV-TXT-COLOR    "#9966CC")
  
    ;; Boolean Symbol -> (union #t error)
  ;; throws an error if the test fails
  (define (test s b)
    (if (not b) (printf "Error in test ~s!\n" s) #t))
 
  
  (define (check-helper str S)
    (if (regexp-match (string-append "^" S) str)
        (substring str (string-length S))
        str))
  ;; String -> String
  (define (check-date str) (check-helper str "The current date is:"))
  ;; Testing
  (test 'check-date1 (equal? (check-date "The current date is:") ""))
  (test 'check-date2 (equal? (check-date "The current date is:1") "1"))
  (test 'check-date3 (equal? (check-date "The current date is:12") "12"))
  (test 'check-date4 (equal? (check-date "The current date is") "The current date is"))
  
    ;; String -> String
  (define (check-time str) (check-helper str "The current time is:"))
  ;; Testing
  (test 'check-time1 (equal? (check-time "The current time is:") ""))
  (test 'check-time2 (equal? (check-time "The current time is:1") "1"))
  (test 'check-time3 (equal? (check-time "The current time is:12") "12"))
  (test 'check-time4 (equal? (check-time "The current time is") "The current time is"))
  
  ;; today : String
  ;; when the script is invoked 
  (define today
    (date->string (seconds->date (current-seconds)) #t))
;    (if (regexp-match "299" (version))
;        (read-line (car (process "date"))) 
;        (string-append
;         (check-date (read-line (car (process "date"))))
;         " "
;         (check-time (read-line (car (process "time")))))))

  )
